@@ -12,7 +12,7 @@
 #
 # written by Jeremy Eglen
 # Created: November 2, 2015
-# Last Modified: February 27, 2020
+# Last Modified: February 12, 2021
 # Originally developed on Python 3.4 and 3.5; this version modified to work with 3.6; should work on any version >3
 # working with Python 3.7 and 3.8
 # don't use Python 2!
@@ -30,7 +30,7 @@ from sparki_learning.util import *
 
 ########### GLOBAL VARIABLES ###########
 # ***** SERIAL TIMEOUT ***** #
-CONN_TIMEOUT = 2  # in seconds
+CONN_TIMEOUT = 1  # in seconds
 
     
 # ***** COMPILE OPTIONS ***** #
@@ -49,7 +49,7 @@ command_queue = []  # this stores every command sent to Sparki
 
 centimeters_moved = 0  # this stores the sum of centimeters moved forward or backward using the moveForwardcm()
 # or moveBackwardcm() functions; used implicitly by moveTo() and moveBy(); use directly
-# by getCentimetersMover(); this only increases in value
+# by getCentimetersMoved(); this only increases in value
 
 current_lcd_color = LCD_BLACK  # this is the color that an LCDdraw command will draw in -- can be LCD_BLACK or LCD_WHITE
 
@@ -66,13 +66,6 @@ init_time = -1  # time when the robot was initialized
 
 robot_library_version = None  # version of the code on the robot
 
-logging.basicConfig(level=logging.DEBUG)  # we want to catch all DEBUG messages
-sparki_logger = logging.getLogger("sparki_learning")
-sparki_logger.setLevel(logging.WARN)
-sparki_std_handler = logging.StreamHandler(sys.stderr)
-sparki_std_handler.setLevel(logging.WARN)
-sparki_logger.addHandler(sparki_std_handler)
-
 serial_port = None  # save the serial port on which we're connected
 serial_conn = None  # hold the pyserial object
 serial_is_connected = False  # set to true once connection is done
@@ -458,6 +451,7 @@ def backward(speed, time=-1):
         nothing
     """
     printDebug("In backward, speed is " + str(speed) + " and time is " + str(time), DEBUG_INFO)
+    speed = float(speed)
 
     # adjust speed to Sparki's requirements
     if speed < 0:
@@ -626,6 +620,7 @@ def forward(speed, time=-1):
         nothing
     """
     printDebug("In forward, speed is " + str(speed) + " and time is " + str(time), DEBUG_INFO)
+    speed = float(speed)
 
     # adjust speed to Sparki's requirements
     if speed < 0:
@@ -1087,7 +1082,7 @@ def gripperStop():
     sendSerial(COMMAND_CODES["GRIPPER_STOP"])
 
 
-def init(com_port, print_versions=True):
+def init(com_port, print_versions=True, auto=False):
     """ Connects to the Sparki robot on com_port; if it is already connected, this will disconnect and reconnect on the given port
         Note that Sparki MUST already be paired with the computer over Bluetooth
         
@@ -1096,9 +1091,10 @@ def init(com_port, print_versions=True):
                    if com_port is the string "mac", this will assume the standard mac port ("/dev/tty.ArcBotics-DevB")
                    if com_port is the string "hc06", this will assume the standard HC-06 port ("tty.HC-06-DevB")
         print_versions - boolean whether or not to print connection message
+        auto - boolean whether this is an auto connection attempt -- True suppresses exceptions
         
         returns:
-        nothing
+        boolean - True if connected, False otherwise
     """
     global init_time
     global robot_library_version, SPARKI_MYRO_VERSION
@@ -1124,9 +1120,12 @@ def init(com_port, print_versions=True):
     try:
         serial_conn = serial.Serial(port=serial_port, baudrate=9600, timeout=CONN_TIMEOUT)
     except serial.SerialException:
-        printUnableToConnect()
-        raise
-
+        if auto:
+            return False
+        else:
+            printUnableToConnect()
+            raise
+        
     serial_is_connected = True  # have to do this prior to sendSerial, or sendSerial will never try to send
 
     sendSerial(COMMAND_CODES["INIT"])
@@ -1155,21 +1154,74 @@ def init(com_port, print_versions=True):
             printDebug(
                 "Unknown library version, using defaults -- you might need an upgrade of the Sparki Learning Python library",
                 DEBUG_ALWAYS)
-            printDebug("(to upgrade the library type: pip sparki-learning --upgrade)", DEBUG_ALWAYS)
+            printDebug("(to upgrade the library type: pip3 sparki-learning --upgrade)", DEBUG_ALWAYS)
             printDebug("Sparki Capabilities will be limited", DEBUG_ALWAYS)
         
         if USE_EEPROM and print_versions:
             robot_name = getName()
             printDebug(robot_name + " is ready", DEBUG_ALWAYS)
+            
+        return True
     else:
-        printDebug("Sparki communication failed", DEBUG_ALWAYS)
+        if not auto:
+            printDebug("Sparki communication failed", DEBUG_ALWAYS)
+            
         serial_is_connected = False
         init_time = -1
+        
+        return False
 
 
 initialize = init # Synonym for init(com_port)
 
 
+def initAuto(print_versions=True, print_correct_port=True):
+    """ Connects to the Sparki robot by guessing the com port; if it is already connected, this will disconnect and reconnect on the given port
+        Note that Sparki MUST already be paired with the computer over Bluetooth for this to work
+        
+        arguments:
+        print_versions - boolean whether or not to print connection message
+        print_correct_port - boolean whether or not to print the correct port
+        
+        returns:
+        string - the com port used to connect
+    """
+    printDebug("In initAuto, print_versions={}; print_correct_port={}".format(print_versions, print_correct_port), DEBUG_INFO)
+
+    possible_ports = [] # store potential ports to try
+    successful_port = None
+    
+    currentOS = platform.system()
+
+    if currentOS == "Darwin":
+        possible_ports = [ "/dev/tty.ArcBotics-DevB", "/dev/tty.HC-06-DevB", "/dev/tty.ArcBotics-SPPDev" ]
+    elif currentOS == "Windows":
+        possible_ports = [ "com" + str(x) for x in range(3,11) ]
+    else:
+        raise RuntimeError("unable to choose the port automatically for your OS")
+
+    for port in possible_ports:
+        success = init(port, print_versions=print_versions, auto=True)
+        
+        if success:
+            successful_port = port
+            
+            if print_correct_port:
+                print("{} is the correct port, which you can use with init() in the future".format(port))
+                
+            break  
+        else:
+            printDebug("{} is not the correct port".format(port), DEBUG_DEBUG)
+            
+    if successful_port is None:
+        printDebug("No Sparki found!", DEBUG_ALWAYS)
+        printDebug("Ports tested: {}".format("; ".join(possible_ports)), DEBUG_ERROR)
+        printUnableToConnect()
+        raise serial.SerialException("No Sparki found on any tested port.")
+
+    return successful_port
+
+
 def isMoving():
     """ Returns True if the robot is in motion
         Note that this can be unreliable - there's no way to "ask" the robot if it is moving
@@ -1585,11 +1637,13 @@ def motors(left_speed, right_speed, time=-1):
         nothing
     """
     global in_motion
-
     printDebug(
         "In motors, left speed is " + str(left_speed) + ", right speed is " + str(right_speed) + " and time is " + str(
             time), DEBUG_INFO)
 
+    left_speed = float(left_speed)
+    right_speed = float(right_speed)
+
     if left_speed == 0 and right_speed == 0:
         printDebug("In motors, both speeds == 0, stopping (but please use stop())", DEBUG_WARN)
         stop()
@@ -2309,6 +2363,8 @@ def syncWait(server_ip=None, server_port=32216):
     from sparki_learning.sync_lib import get_client_start
     wait_time = get_client_start(server_ip, server_port)
 
+    print("Waiting for {} seconds".format(wait_time))
+
     waitNoop(wait_time)
 
 
@@ -2333,6 +2389,7 @@ def turnBy(degrees):
 
     printDebug("In turnBy, degrees is " + str(degrees), DEBUG_INFO)
 
+    degrees = float(degrees)
     degrees = wrapAngle(degrees)
 
     if abs(degrees) >= 360:  # >= in case there's a rounding error
@@ -2392,6 +2449,8 @@ def turnLeft(speed, time=-1):
     """
     printDebug("In turnLeft, speed is " + str(speed) + " and time is " + str(time), DEBUG_INFO)
 
+    speed = float(speed)
+
     # adjust speed to Sparki's requirements
     if speed < 0:
         printDebug("In turnLeft, speed < 0, calling turnRight", DEBUG_WARN)
@@ -2419,6 +2478,8 @@ def turnRight(speed, time=-1):
     """
     printDebug("In turnRight, speed is " + str(speed) + " and time is " + str(time), DEBUG_INFO)
 
+    speed = float(speed)
+
     # adjust speed to Sparki's requirements
     if speed < 0:
         printDebug("In turnRight, speed < 0, calling turnLeft", DEBUG_WARN)
@@ -2444,6 +2505,7 @@ def wait(wait_time):
         nothing
     """
     printDebug("In wait, wait_time is " + str(wait_time), DEBUG_INFO)
+    wait_time = float(wait_time)
     maxWait = 600
 
     if wait_time >= maxWait:
@@ -2468,6 +2530,7 @@ def waitNoop(wait_time):
         nothing
     """
     printDebug("In waitNoop, wait_time is " + str(wait_time), DEBUG_INFO)
+    wait_time = float(wait_time)
     maxWait = 1200
     sleepTime = 1
 
